@@ -12,18 +12,18 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, see <http://www.gnu.org/licenses/>.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  */
 #ifndef _MYATT_COMPILER_
 #define _MYATT_COMPILER_
 
 #include <string>
 #include <fstream>
-#include <sstream>
 #include <map>
 #include <set>
 #include <vector>
 
+#include <lttoolbox/ustring.h>
 #include <lttoolbox/alphabet.h>
 #include <lttoolbox/transducer.h>
 #include <lttoolbox/compression.h>
@@ -36,25 +36,11 @@
 #define BOTH      3
 
 using namespace std;
+using namespace icu;
 
 /** Bitmask; 1 = WORD, 2 = PUNCT, 3 = BOTH. */
 typedef unsigned int TransducerType;
 
-namespace
-{
-  /** Splits a string into fields. */
-  inline vector<wstring>& split(const wstring& s, wchar_t delim, vector<wstring> &out)
-  {
-      wistringstream ss(s);
-      wstring item;
-      while (getline(ss, item, delim))
-      {
-        out.push_back(item);
-      }
-      return out;
-  }
-};
-
 /**
  * Converts transducers from AT&T text format to lt binary format.
  *
@@ -87,64 +73,56 @@ public:
                            Transducer& transducer, map<int, int>& corr,
                            set<int>& visited) ;
 
-  /**
-   * Classifies the edges of the transducer graphs recursively. It works like
-   * this:
-   * - the type of the starting state is BOTH (already set)
-   * - in case of an epsilon move, the type of the target state is the same as
-   *   that of the source
-   * - the first non-epsilon transition determines the type of the whole path
-   * - it is also the time from which we begin filling the @p visited set.
-   *
-   * @param from the id of the source state.
-   * @param visited the ids of states visited by this path.
-   * @param path are we in a path?
-   */
-  void classify(int from, map<int, TransducerType>& visited, bool path,
-                TransducerType type) ;
-
-
-
   /**
    * Reads the AT&T format file @p file_name. The transducer and the alphabet
    * are both cleared before reading the new file.
+   * If read_rl = true then the second tape is used as the input
    */
-  void parse(string const &file_name, wstring const &dir);
+  void parse(string const &file_name, bool read_rl);
 
   /** Writes the transducer to @p file_name in lt binary format. */
 
   void write(FILE *fd) ;
 
+  void setHfstSymbols(bool b);
+  void setSplitting(bool b);
+
 private:
 
+  bool hfstSymbols = false;
+  bool splitting = true;
+
   /** The final state(s). */
   map<int, double> finals;
   /**
    * Id of the starting state. We assume it is the source state of the first
    * transduction in the file.
    */
-  int starting_state;
+  int starting_state = 0;
   /**
    * Default value of weight of a transduction unless specified.
    */
-  double default_weight;
+  double default_weight = 0.0000;
+
+  // how many phantom states have we created to split multichar symbols
+  int phantom_count = 0;
 
   Alphabet alphabet;
   /** All non-multicharacter symbols. */
-  set<wchar_t> letters;
+  set<UChar> letters;
 
   /** Used in AttNode. */
   struct Transduction
   {
     int            to;
-    wstring        upper;
-    wstring        lower;
+    UString        upper;
+    UString        lower;
     int            tag;
     double         weight;
     TransducerType type;
 
-    Transduction(int to, wstring upper, wstring lower, int tag, double weight,
-                 TransducerType type=UNDECIDED) :
+    Transduction(int to, UString upper, UString lower, int tag,
+                 double weight, TransducerType type=UNDECIDED) :
       to(to), upper(upper), lower(lower), tag(tag), weight(weight), type(type) {}
   };
 
@@ -188,30 +166,31 @@ private:
    * Returns true for combining diacritics and modifier letters
    *
    */
-  bool is_word_punct(wchar_t symbol);
+  bool is_word_punct(UChar32 symbol);
+
+  /**
+   * Determines initial type of single transition
+   *
+   */
+  void classify_single_transition(Transduction& t);
+
+  void classify_forwards();
+  TransducerType classify_backwards(int state, set<int>& path);
 
   /**
    * Converts symbols like @0@ to epsilon, @_SPACE_@ to space, etc.
    * @todo Are there other special symbols? If so, add them, and maybe use a map
    *       for conversion?
    */
-  void convert_hfst(wstring& symbol);
+  void convert_hfst(UString& symbol);
 
-  /**
-   * Returns the code of the symbol in the alphabet. Run after convert_hfst has
-   * run.
-   *
-   * Also adds all non-multicharacter symbols (letters) to the @p letters set.
-   *
-   * @return the code of the symbol, if @p symbol is multichar; its first (and
-   *         only) character otherwise.
-   */
-  int symbol_code(const wstring& symbol);
-
-  /**
-   * Finds whether an at&t file contains multiple FSTs or not
-  */
-  bool has_multiple_fsts(string const &file_name);
+  // if a character should be in the alphabet, add it
+  void update_alphabet(UChar32 c);
+  // convert a string to a symbol code, splitting non-tag multichars
+  void symbol_code(const UString& symbol, vector<int32_t>& split);
+  void add_transition(int from, int to,
+                      const UString& upper, const UString& lower,
+                      double weight);
 };
 
 #endif /* _MYATT_COMPILER_ */
