@@ -12,7 +12,7 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, see <http://www.gnu.org/licenses/>.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  */
 #ifndef _BUFFER_
 #define _BUFFER_
@@ -34,23 +34,23 @@ private:
    * Buffer size.
    */
   unsigned int size;
-  
+
   /**
    * Buffer array.
    */
   T *buf;
-   
+
   /**
    * Buffer current position.
    */
   unsigned int currentpos;
-  
+
   /**
    * Last position.
    */
   unsigned int lastpos;
-  
-  
+
+
   void copy(Buffer const &b)
     {
       currentpos = b.currentpos;
@@ -65,7 +65,7 @@ private:
       delete[] buf;
     }
 
-public:  
+public:
 
   /**
    * Constructor
@@ -75,9 +75,9 @@ public:
     {
       if(buf_size == 0)
       {
-	cerr << "Error: Cannot create empty buffer." << endl;
-	exit(EXIT_FAILURE);
-      }    
+        cerr << "Error: Cannot create empty buffer." << endl;
+        exit(EXIT_FAILURE);
+      }
       buf = new T[buf_size];
       size = buf_size;
       currentpos = 0;
@@ -115,12 +115,12 @@ public:
     {
       if(&b != this)
       {
-	destroy();
-	copy(b);
+        destroy();
+        copy(b);
       }
       return *this;
     }
-    
+
   /**
    * Add an element to the buffer.
    * @param value the value.
@@ -130,13 +130,13 @@ public:
     {
       if(lastpos == size)
       {
-	lastpos = 0;
+        lastpos = 0;
       }
       buf[lastpos++] = value;
       currentpos = lastpos;
       return buf[lastpos - 1];
     }
-  
+
 
   /**
    * Add an element to the buffer and not modify its content, getting the
@@ -147,7 +147,7 @@ public:
     {
       if(lastpos == size)
       {
-	lastpos = 0;
+        lastpos = 0;
       }
       currentpos = lastpos;
       return buf[lastpos -1];
@@ -162,18 +162,40 @@ public:
     {
       if(currentpos != lastpos)
       {
-	if(currentpos == size)
-	{
-	  currentpos = 0;
-	}
-	return buf[currentpos++];
+        if(currentpos == size)
+          {
+            currentpos = 0;
+          }
+        return buf[currentpos++];
       }
       else
       {
-	return last();
+        return last();
       }
     }
-    
+
+  /**
+   * Look at the current element of the buffer without advancing.
+   * @return current element.
+   */
+  T & peek() const
+    {
+      if(currentpos != lastpos)
+      {
+        if(currentpos == size)
+        {
+          return buf[0];
+        }
+        else {
+          return buf[currentpos];
+        }
+      }
+      else
+      {
+        return last();
+      }
+    }
+
   /**
    * Get the last element of the buffer.
    * @return last element.
@@ -182,14 +204,15 @@ public:
     {
       if(lastpos != 0)
       {
-	return buf[lastpos-1];
+        return buf[lastpos-1];
       }
       else
       {
-	return buf[size-1];
+        return buf[size-1];
       }
     }
- 
+
+
   /**
    * Get the current buffer position.
    * @return the position.
@@ -218,13 +241,13 @@ public:
     {
       if(prevpos <= currentpos)
       {
-	return currentpos - prevpos;
+        return currentpos - prevpos;
       }
       else
       {
-	return currentpos + size - prevpos;
+        return currentpos + size - prevpos;
       }
-    } 
+    }
 
   /**
    * Return the range size between the buffer current position and a
@@ -236,11 +259,11 @@ public:
     {
       if(postpos >= currentpos)
       {
-	return postpos - currentpos;
+        return postpos - currentpos;
       }
       else
       {
-	return postpos + size - currentpos;
+        return postpos + size - currentpos;
       }
     }
 
@@ -250,9 +273,9 @@ public:
    */
   bool isEmpty() const
     {
-      return currentpos == lastpos;   
+      return currentpos == lastpos;
     }
-  
+
   /**
    * Gets back 'posback' positions in the buffer.
    * @param posback the amount of position to get back.
