@@ -12,9 +12,10 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, see <http://www.gnu.org/licenses/>.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  */
 #include <lttoolbox/lt_locale.h>
+#include <unicode/uloc.h>
 
 #include <clocale>
 #include <iostream>
@@ -35,13 +36,16 @@ LtLocale::tryToSetLocale()
     // Nothing
   }
 
+  UErrorCode status = U_ZERO_ERROR;
+  uloc_setDefault("en_US_POSIX", &status);
+
 #if !defined(__CYGWIN__) && !defined (__MINGW32__)
   if(setlocale(LC_CTYPE, "") != NULL)
   {
     return;
   }
 
-  wcerr << "Warning: unsupported locale, fallback to \"C\"" << endl;
+  cerr << "Warning: unsupported locale, fallback to \"C\"" << endl;
 
   setlocale(LC_ALL, "C");
 #endif
