@@ -11,7 +11,7 @@
 // General Public License for more details.
 //
 // You should have received a copy of the GNU General Public License
-// along with this program; if not, see <http://www.gnu.org/licenses/>.
+// along with this program; if not, see <https://www.gnu.org/licenses/>.
 
 #ifndef SERIALISER_H
 #define SERIALISER_H
@@ -29,6 +29,7 @@
 #include <string>
 #include <utility>
 #include <vector>
+#include <unicode/uchar.h>
 
 namespace {
 template <typename SerialisedType>
@@ -96,18 +97,18 @@ public:
                                std::ostream &Output);
 };
 
-template <> class Serialiser<wchar_t> {
-public:
-  inline static void serialise(const wchar_t &SerialisedType_,
-                               std::ostream &Output);
-};
-
 template <> class Serialiser<char> {
 public:
   inline static void serialise(const char &SerialisedType_,
                                std::ostream &Output);
 };
 
+template <> class Serialiser<UChar> {
+public:
+  inline static void serialise(const UChar &SerialisedType_,
+                               std::ostream &Output);
+};
+
 template<> class Serialiser<double> {
 public:
   inline static void serialise(const double &SerialisedType_,
@@ -213,16 +214,16 @@ void Serialiser<uint32_t>::serialise(const uint32_t &SerialisedType_,
   int_serialise((uint64_t)SerialisedType_, Output);
 }
 
-void Serialiser<wchar_t>::serialise(const wchar_t &SerialisedType_,
-                                    std::ostream &Output) {
-  int_serialise((uint32_t)SerialisedType_, Output);
-}
-
 void Serialiser<char>::serialise(const char &SerialisedType_,
                                  std::ostream &Output) {
   int_serialise((uint8_t)SerialisedType_, Output);
 }
 
+void Serialiser<UChar>::serialise(const UChar &SerialisedType_,
+                                  std::ostream &Output) {
+  int_serialise((uint16_t)SerialisedType_, Output);
+}
+
 void Serialiser<double>::serialise(const double &SerialisedType_,
                                    std::ostream &Output) {
   union {
