@@ -12,7 +12,7 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, see <http://www.gnu.org/licenses/>.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  */
 #ifndef _TRANSDUCTOR_
 #define _TRANSDUCTOR_
@@ -87,10 +87,26 @@ private:
    * @param symbol the string to be escaped
    * @param hfst if true, use HFST-compatible escape sequences
    */
-  void escapeSymbol(wstring& symbol, bool hfst) const;
+  void escapeSymbol(UString& symbol, bool hfst) const;
 
 public:
 
+  /**
+   * String constants
+   */
+  static UString const HFST_EPSILON_SYMBOL_SHORT;
+  static UString const HFST_EPSILON_SYMBOL_LONG;
+  static UString const LTTB_EPSILON_SYMBOL;
+  static UString const HFST_SPACE_SYMBOL;
+  static UString const HFST_TAB_SYMBOL;
+  static UString const GROUP_SYMBOL;
+  static UString const JOIN_SYMBOL;
+  static UString const ANY_TAG_SYMBOL;
+  static UString const ANY_CHAR_SYMBOL;
+  static UString const LSX_BOUNDARY_SYMBOL;
+  static UString const COMPOUND_ONLY_L_SYMBOL;
+  static UString const COMPOUND_R_SYMBOL;
+
   /**
    * Constructor
    */
@@ -172,7 +188,7 @@ public:
    * @param a widestring of the pattern to be recognised
    * @return true if the pattern is recognised by the transducer
    */
-  bool recognise(wstring pattern, Alphabet &a, FILE *err = stderr);
+  bool recognise(UString pattern, Alphabet &a, FILE *err = stderr);
 
   /**
    * Set the state as a final or not, yes by default
@@ -232,8 +248,8 @@ public:
    * @param hfst if true, use HFST-compatible escape characters
    * @param epsilon_tag the tag to take as epsilon
    */
-  void show(Alphabet const &a, FILE *output = stdout, int const epsilon_tag = 0, bool hfst = false) const;
-  void show(Alphabet const &a, FILE *output = stdout, int const epsilon_tag = 0) const;
+  void show(Alphabet const &a, UFILE *output, int const epsilon_tag = 0, bool hfst = false) const;
+  void show(Alphabet const &a, UFILE *output, int const epsilon_tag = 0) const;
 
   /**
    * Determinize the transducer
@@ -400,6 +416,13 @@ public:
                        Alphabet const &t_a,
                        int const epsilon_tag = 0);
 
+  /**
+   * Ensure that new_alpha contains all the symbols in old_alpha
+   * and update all transitions to the symbol numbers of new_alpha.
+   * If has_pairs is false, transition labels will be looked up as
+   * single symbols rather than pairs.
+   */
+  void updateAlphabet(Alphabet& old_alpha, Alphabet& new_alpha, bool has_pairs = true);
 };
 
 #endif
