@@ -12,15 +12,17 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, see <http://www.gnu.org/licenses/>.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  */
 #ifndef _XMLPARSEUTIL_
 #define _XMLPARSEUTIL_
 
 #include <libxml/encoding.h>
 #include <libxml/xmlreader.h>
+#include <lttoolbox/ustring.h>
+#include <vector>
+#include <cstdint>
 #include <string>
-#include <cwchar>
 
 using namespace std;
 
@@ -29,14 +31,16 @@ class XMLParseUtil
 public:
 
   /* If attrib does not exist (or other error), returns an empty string: */
-  static wstring attrib(xmlTextReaderPtr reader, wstring const &name);
+  static UString attrib(xmlTextReaderPtr reader, UString const &name);
 
   /* If attrib does not exist (or other error), returns fallback: */
-  static wstring attrib(xmlTextReaderPtr reader, wstring const &name, const wstring fallback);
+  static UString attrib(xmlTextReaderPtr reader, UString const &name, const UString& fallback);
 
-  static string latin1(xmlChar const * input); // mark for deletion
-  static wstring towstring(xmlChar const * input);
-  static wstring stows(string const &str);
+  static string attrib_str(xmlTextReaderPtr reader, const UString& name);
+
+  static UString readName(xmlTextReaderPtr reader);
+  static UString readValue(xmlTextReaderPtr reader);
+  static void readValueInto32(xmlTextReaderPtr reader, vector<int32_t>& vec);
 };
 
 #endif
