@@ -24,6 +24,11 @@ public:
   bool operator==(const children& other) const;
 };
 
+xmlNode* load_xml(const char* fname);
+void error_and_die(xmlNode* node, const char* fmt, ...);
+
 UString getattr(xmlNode* node, const char* attr);
+UString getattr(xmlNode* node, const UString& attr, const UString& fallback);
+UString getattr(xmlNode* node, const UString& attr);
 
 #endif
