@@ -1,22 +1,24 @@
 # -*- coding: utf-8 -*-
 
-import itertools
+import os
 from subprocess import Popen, PIPE, call
 from tempfile import mkdtemp
 from shutil import rmtree
 from basictest import BasicTest
 
+
 class PrintTest(BasicTest):
     """See lt_print test for how to use this. Override runTest if you don't
     want to use NUL flushing."""
 
     printdix = "data/minimal-mono.dix"
     printdir = "lr"
-    expectedOutput = itertools.repeat("")
+    expectedOutput = ""
     expectedRetCodeFail = False
+    printflags = []
 
     def compileTest(self, tmpd):
-        self.assertEqual(0, call(["../lttoolbox/lt-comp",
+        self.assertEqual(0, call([os.environ['LTTOOLBOX_PATH']+"/lt-comp",
                                   self.printdir,
                                   self.printdix,
                                   tmpd+"/compiled.bin"],
@@ -26,7 +28,7 @@ class PrintTest(BasicTest):
         tmpd = mkdtemp()
         try:
             self.compileTest(tmpd)
-            self.printresult = Popen(["../lttoolbox/lt-print"] + [tmpd+"/compiled.bin"],
+            self.printresult = Popen([os.environ['LTTOOLBOX_PATH']+"/lt-print"] + self.printflags + [tmpd+"/compiled.bin"],
                               stdout=PIPE,
                               stderr=PIPE)
 
