@@ -9,6 +9,7 @@ import lt_proc
 import lt_trim
 import lt_print
 import lt_comp
+import lt_append
 
 os.environ['LTTOOLBOX_PATH'] = '../lttoolbox'
 if len(sys.argv) > 1:
@@ -17,7 +18,7 @@ if len(sys.argv) > 1:
 if __name__ == "__main__":
     os.chdir(os.path.dirname(__file__))
     failures = 0
-    for module in [lt_trim, lt_proc, lt_print, lt_comp]:
+    for module in [lt_trim, lt_proc, lt_print, lt_comp, lt_append]:
         suite = unittest.TestLoader().loadTestsFromModule(module)
         res = unittest.TextTestRunner(verbosity = 2).run(suite)
         failures += len(res.failures)
